@@ -23,6 +23,7 @@ from .token import TOKEN_FILTER_CONTEXT
 from .token import TOKEN_FILTER_END
 from .token import TOKEN_FILTER_START
 from .token import TOKEN_FLOAT
+from .token import TOKEN_FUNCTION
 from .token import TOKEN_GE
 from .token import TOKEN_GT
 from .token import TOKEN_ILLEGAL
@@ -32,7 +33,6 @@ from .token import TOKEN_INT
 from .token import TOKEN_INTERSECTION
 from .token import TOKEN_LE
 from .token import TOKEN_LG
-from .token import TOKEN_LIST_END
 from .token import TOKEN_LIST_SLICE
 from .token import TOKEN_LIST_START
 from .token import TOKEN_LPAREN
@@ -45,6 +45,7 @@ from .token import TOKEN_NOT
 from .token import TOKEN_NULL
 from .token import TOKEN_OR
 from .token import TOKEN_PROPERTY
+from .token import TOKEN_RBRACKET
 from .token import TOKEN_RE
 from .token import TOKEN_RE_FLAGS
 from .token import TOKEN_RE_PATTERN
@@ -120,6 +121,9 @@ class Lexer:
         # /pattern/ or /pattern/flags
         self.re_pattern = r"/(?P<G_RE>.+?)/(?P<G_RE_FLAGS>[aims]*)"
 
+        # func(
+        self.function_pattern = r"(?P<G_FUNC>[a-z][a-z_0-9]+)\(\s*"
+
         self.rules = self.compile_rules()
 
     def compile_rules(self) -> Pattern[str]:
@@ -133,8 +137,9 @@ class Lexer:
             (TOKEN_SLICE, self.slice_pattern),
             (TOKEN_WILD, self.wild_pattern),
             (TOKEN_LIST_SLICE, self.slice_list_pattern),
-            (TOKEN_FILTER_START, r"\[\s*\?\s*\("),
+            (TOKEN_FILTER_START, r"\[\s*\?\s*\(?"),
             (TOKEN_FILTER_END, r"\)\s*]"),
+            (TOKEN_FUNCTION, self.function_pattern),
             (TOKEN_BRACKET_PROPERTY, self.bracketed_property_pattern),
             (TOKEN_DOT_PROPERTY, self.dot_property_pattern),
             (TOKEN_FLOAT, r"-?\d+\.\d*(?:e[+-]?\d+)?"),
@@ -157,7 +162,7 @@ class Lexer:
             (TOKEN_UNDEFINED, r"undefined"),
             (TOKEN_MISSING, r"missing"),
             (TOKEN_LIST_START, r"\["),
-            (TOKEN_LIST_END, r"]"),
+            (TOKEN_RBRACKET, r"]"),
             (TOKEN_COMMA, r","),
             (TOKEN_EQ, r"=="),
             (TOKEN_NE, r"!="),
@@ -279,6 +284,12 @@ class Lexer:
                     value=match.group(),
                     index=match.start(),
                 )
+            elif kind == TOKEN_FUNCTION:
+                yield _token(
+                    kind=TOKEN_FUNCTION,
+                    value=match.group("G_FUNC"),
+                    index=match.start("G_FUNC"),
+                )
             elif kind == TOKEN_SKIP:
                 continue
             elif kind == TOKEN_ILLEGAL:
