@@ -1,4 +1,4 @@
-""""""
+"""JSONPath tokens."""
 import sys
 from typing import Tuple
 
@@ -19,7 +19,7 @@ TOKEN_FILTER_END = sys.intern("FILTER_END")
 TOKEN_FILTER_START = sys.intern("FILTER_START")
 TOKEN_IDENT = sys.intern("IDENT")
 TOKEN_INDEX = sys.intern("IDX")
-TOKEN_LIST_END = sys.intern("RBRACKET")
+TOKEN_RBRACKET = sys.intern("RBRACKET")
 TOKEN_BARE_PROPERTY = sys.intern("BARE_PROPERTY")
 TOKEN_LIST_SLICE = sys.intern("LSLICE")
 TOKEN_LIST_START = sys.intern("LBRACKET")
@@ -38,6 +38,7 @@ TOKEN_AND = sys.intern("AND")
 TOKEN_BLANK = sys.intern("BLANK")
 TOKEN_CONTAINS = sys.intern("CONTAINS")
 TOKEN_FILTER_CONTEXT = sys.intern("FILTER_CONTEXT")
+TOKEN_FUNCTION = sys.intern("FUNCTION")
 TOKEN_EMPTY = sys.intern("EMPTY")
 TOKEN_EQ = sys.intern("EQ")
 TOKEN_FALSE = sys.intern("FALSE")
@@ -75,7 +76,7 @@ TOKEN_INTERSECTION = sys.intern("INTERSECT")
 
 
 class Token:
-    """A token, as returned from :meth:`lex.Lexer.tokenize`."""
+    """A token, as returned from `lex.Lexer.tokenize()`."""
 
     __slots__ = ("kind", "value", "index", "path")
 
